#include "timer.h"

char	sbuf[15];
char	line[12];
char	starburst[DISPLAY_DIGITS];
unsigned char 	dpos;
unsigned char	pulses, sec, min, hour, t_track;

#define smess	(char*)(&sbuf[SCROLL_BUF_LEN])

void clock(void);
void cput(unsigned char c);
void scroll(const char *mess, unsigned char ntimes);
void display(const char *message);
void clrscr(void);
char choice(const char *const *menu);
char getch(void);
void delay(unsigned char time); 
char choice(const char *const *fmenu);

/************************************************************************************/
/* cput()                                                                           */
/* This routine writes a single byte to the starburst display unit.                 */
/************************************************************************************/

void cput(unsigned char c) 
{
    unsigned char	addr;
    
	TRISB=0x00;								/* port B is now data output to display */
	TRISA=0x00;								/* port RA4:RA3 is output control RA2:RA0 is output for address */
	addr=SCROLL_BUF_LEN-dpos++;				/* address of segment */
	PORTA=(PORTA&0x18)|(addr&0x03);			/* RA0, RA1 are address of display cell*/
	PORTB=(PORTB&0x80)|(0x7F&c);			/* character to be written */
	if(addr&0x04) {							/* 1st four characters */
		DISPLAY_WR1();						/* write pulse */
	} else {								/* 2nd four characters */
		DISPLAY_WR0();						/* write pulse */
	}
	starburst[SCROLL_BUF_LEN-addr]=c;		/* debugging remove later */
	TRISA|=0x07;							/* port A is now input */
	TRISB=0x7F;								/* port RB7 is output RB6:RB0 is now input */
}

/************************************************************************************/
/* display()                                                                        */
/* This routine writes a string of eight characters to the starburst display unit.  */
/************************************************************************************/

void display(const char *message)
{
    unsigned char    i,c;
    
    dpos=0;									/* first display position */
    for(i=0; i<DISPLAY_DIGITS; i++) { 		/* DISPLAY_DIGITS positions to update */
        if(c=*message++) {					/* if part of message */
            cput(c);						/* print string until terminator found */
        } else {
            while(i++<DISPLAY_DIGITS) {		/* rest of display area fill with spaces */
                cput(' ');					/* pad space character for rest of display */
            }
        }
    } 
}

/************************************************************************************/
/* getch()                                                                          */
/* This routine reads the pushbutton state and returns the value   .                */
/************************************************************************************/
char getch(void)
{
	return PORTA&0x07;						/* read the 3 pushbuttons state */
}

/************************************************************************************/
/* clrdisplay()                                                                     */
/* This routine clears the starburst display screen                                 */
/************************************************************************************/

void clrscr(void) 
{
    display("");                          	/* a clear display */
    dpos=0;                                 /* reset display position */
}

/************************************************************************************/
/* scroll()                                                                         */
/* This provides a scrolling message a number of times                              */
/************************************************************************************/

void scroll(const char *mess, unsigned char ntimes)
{
	unsigned char	stop;

	memset(sbuf,' ',SCROLL_BUF_LEN);
	strcpy(smess,mess);                  	/* copy message into buffer */
	do {
		mess=sbuf;                          /* point to begin of message */
		do {
            display(mess++);                /* display part of the message */
			stop=pulses+Delay500ms;
            do {
				if(kbhit()) {				/* abort scrolling ? */
    				clrscr();      			/* a clear display */
        	    	return;       			/* yes then bye */
   				}         
			} while(stop!=pulses);
        } while(*mess);                     /* until all characters are displayed */
    } while(--ntimes);                      /* until message is displayed ntimes */
    clrscr();                  				/* a clear display */
}

/************************************************************************************/
/* choice()                                                                         */
/* This a scrolling menu from which user functions can be selected for use          */
/************************************************************************************/

char choice(const	char *const *menu)
{
    const 	char    *src;
    char	i,j,stop;

    for(i=0;i<3;i++) {
        for(j=0;src=menu[j];j++) {         		/* get next string to be scrolled */ 
            scroll(src,1);                  	/* scroll the menu item message */
			stop=pulses+Delay500ms;
			do {
				if(kbhit()) {                  	/* check for keypad activities */
					clrscr();					/* clear display */
   		         	return j;	             	/* if yes return with the key code */
				}
			} while(stop!=pulses);
        }
    }
    scroll("--TIMEOUT--",1);                	/* no input for 20 seconds stop processing */
    return(0xFF);                          		/* dummy return will never be taken */
}

void delay(unsigned char time)
{
	unsigned char	stop;

	stop=pulses+time;							/* stop when stop marker is reached */
	while(stop!=pulses) ;						/* other wait for stop condition */
}

interrupt isr(void)
{
	if(T0IF) {
		++pulses;
		++t_track;								// increment pulses
	if(t_track==160){
		clock();
		t_track = 0;
			}	
		T0IF=0;									// reset TMR0 flag
		}	
		
	

}

void clock(void)
{
	++sec;
	if(++sec==60){
           min++;
           sec = 0; }
	if(++min==60){
           hour++; }
	if(++hour==24){
        hour=0; }
}
